@@ -3,7 +3,6 @@ from __future__ import division, print_function
 import numpy as np
 import vaex
 from .dataset import Task, TaskMapReduce, _parse_f
-import scipy
 import six
 
 def _asfloat(a):
@@ -56,7 +55,7 @@ class TaskHistogram(Task):
 		info.i1 = i1
 		info.i2 = i2
 		info.first = i1 == 0
-		info.last = i2 == self.dataset.full_length()
+		info.last = i2 == self.dataset.length_unfiltered()
 		info.size = i2-i1
 		#print "bin", i1, i2, info.last
 		#self.grids["counts"].bin_block(info, *blocks)
@@ -65,7 +64,7 @@ class TaskHistogram(Task):
 
 		blocks = [_asfloat(block) for block in blocks]
 
-		if self.masked or self.dataset.selection_global:
+		if self.masked or self.dataset.filtered:
 			mask = self.dataset.evaluate_selection_mask("default" if self.masked else None, i1=i1, i2=i2)
 			blocks = [block[mask] for block in blocks]
 
@@ -116,6 +115,7 @@ class SubspaceGridded(object):
 		return SubspaceGridded(self.subspace_bounded, self.grid, vx=vx, vy=vy, vcounts=counts)
 
 	def filter_gaussian(self, sigmas=1):
+		import scipy.ndimage
 		return SubspaceGridded(self.subspace_bounded, scipy.ndimage.filters.gaussian_filter(self.grid, sigmas))
 
 	def clip_relative(self, v1, v2):
@@ -269,17 +269,17 @@ class Subspaces(object):
 		self.subspaces = subspaces
 		self.expressions = set()
 		first_subspace = self.subspaces[0]
-		self.async = first_subspace.async
+		self.delay = first_subspace.delay
 		self.dimension = first_subspace.dimension
 		self.dataset = self.subspaces[0].dataset
 		for subspace in self.subspaces:
 			assert subspace.dataset == self.subspaces[0].dataset
-			assert subspace.async == self.subspaces[0].async
+			assert subspace.delay == self.subspaces[0].delay
 			assert subspace.dimension == self.subspaces[0].dimension, "subspace is of dimension %s, while first subspace if of dimension %s" % (subspace.dimension, self.subspaces[0].dimension)
-			#assert subspace.sele== self.subspaces[0].async
+			#assert subspace.sele== self.subspaces[0].delay
 			self.expressions.update(subspace.expressions)
 		self.expressions = list(self.expressions)
-		self.subspace = self.dataset(*list(self.expressions), async=self.async, executor=first_subspace.executor)
+		self.subspace = self.dataset(*list(self.expressions), delay=self.delay, executor=first_subspace.executor)
 
 	#def _repr_html_(self):
 
@@ -316,13 +316,13 @@ class Subspaces(object):
 
 
 	def minmax(self):
-		if self.async:
+		if self.delay:
 			return self.subspace.minmax().then(self._unpack)
 		else:
 			return self._unpack(self.subspace.minmax())
 
 	def limits_sigma(self, sigmas=3, square=False):
-		if self.async:
+		if self.delay:
 			return self.subspace.limits_sigma(sigmas=sigmas, square=square).then(self._unpack)
 		else:
 			return self._unpack(self.subspace.limits_sigma(sigmas=sigmas, square=square))
@@ -339,12 +339,12 @@ class Subspaces(object):
 			limits_promise = vaex.promise.Promise.fulfilled(limits)
 		limits_promise = limits_promise.then(self._unpack)
 		promise = limits_promise.then(mutual_information)
-		return promise if self.async else promise.get()
+		return promise if self.delay else promise.get()
 
 
 
 	def mean(self):
-		if self.async:
+		if self.delay:
 			return self.subspace.mean().then(self._unpack)
 		else:
 			means = self.subspace.mean()
@@ -356,7 +356,7 @@ class Subspaces(object):
 			means = self._pack(means)
 		def var(means):
 			return self.subspace.var(means=means)
-		if self.async:
+		if self.delay:
 			#if means is None:
 			#	return self.subspace.mean().then(var).then(self._unpack)
 			#else:
@@ -379,7 +379,7 @@ class Subspaces(object):
 			means = self._pack(means)
 		if vars is not None:
 			vars = self._pack(vars)
-		if self.async:
+		if self.delay:
 			if means is None:
 				mean_promise = self.subspace.mean()
 			else:
@@ -418,25 +418,25 @@ class Subspace(object):
 	See `vaex.dataset.Dataset` for more documentation.
 
 	"""
-	def __init__(self, dataset, expressions, executor, async, masked=False):
+	def __init__(self, dataset, expressions, executor, delay, masked=False):
 		"""
 
 		:param Dataset dataset: the dataset the subspace refers to
 		:param list[str] expressions: list of expressions that forms the subspace
 		:param Executor executor: responsible for executing the tasks
-		:param bool async: return answers directly, or as a promise
+		:param bool delay: return answers directly, or as a promise
 		:param bool masked: work on the selection or not
 		:return:
 		"""
 		self.dataset = dataset
 		self.expressions = expressions
 		self.executor = executor
-		self.async = async
+		self.delay = delay
 		self.is_masked = masked
 
 	def __repr__(self):
 		name = self.__class__.__module__ + "." +self.__class__.__name__
-		return "<%s(dataset=%r, expressions=%r, async=%r, is_masked=%r)> instance at 0x%x" % (name, self.dataset, self.expressions, self.async, self.is_masked, id(self))
+		return "<%s(dataset=%r, expressions=%r, delay=%r, is_masked=%r)> instance at 0x%x" % (name, self.dataset, self.expressions, self.delay, self.is_masked, id(self))
 
 	@property
 	def dimension(self):
@@ -449,10 +449,10 @@ class Subspace(object):
 		return self.is_masked
 
 	def selected(self):
-		return self.__class__(self.dataset, expressions=self.expressions, executor=self.executor, async=self.async, masked=True)
+		return self.__class__(self.dataset, expressions=self.expressions, executor=self.executor, delay=self.delay, masked=True)
 
-	def asynchronous(self):
-		return self.__class__(self.dataset, expressions=self.expressions, executor=self.executor, async=True, masked=self.is_masked)
+	def delayhronous(self):
+		return self.__class__(self.dataset, expressions=self.expressions, executor=self.executor, delay=True, masked=self.is_masked)
 
 	def image_rgba_save(self, filename, data=None, rgba8=None, **kwargs):
 		if rgba8 is not None:
@@ -1034,7 +1034,7 @@ class SubspaceLocal(Subspace):
 
 	def _task(self, task, progressbar=False):
 		"""Helper function for returning tasks results, result when immediate is True, otherwise the task itself, which is a promise"""
-		if self.async:
+		if self.delay:
 			# should return a task or a promise nesting it
 			return self.executor.schedule(task)
 		else:
@@ -1071,7 +1071,7 @@ class SubspaceLocal(Subspace):
 				result.append((min(min1, min2), max(max1, max2)))
 			return result
 		def min_max_map(thread_index, i1, i2, *blocks):
-			if self.is_masked or self.dataset.selection_global:
+			if self.is_masked or self.dataset.filtered:
 				mask = self.dataset.evaluate_selection_mask("default" if self.is_masked else None, i1=i1, i2=i2)
 				blocks = [block[mask] for block in blocks]
 				is_empty = all(~mask)
@@ -1106,7 +1106,7 @@ class SubspaceLocal(Subspace):
 		def remove_counts(means_and_counts):
 			return self._toarray(means_and_counts)[:,0]
 		def mean_map(thread_index, i1, i2, *blocks):
-			if self.is_masked or self.dataset.selection_global:
+			if self.is_masked or self.dataset.filtered:
 				mask = self.dataset.evaluate_selection_mask("default" if self.is_masked else None, i1=i1, i2=i2)
 				return [(np.nanmean(block[mask]**moment), np.count_nonzero(~np.isnan(block[mask]))) for block in blocks]
 			else:
@@ -1123,7 +1123,7 @@ class SubspaceLocal(Subspace):
 			return vars_and_counts
 		def remove_counts(vars_and_counts):
 			return self._toarray(vars_and_counts)[:,0]
-		if self.is_masked or self.dataset.selection_global:
+		if self.is_masked or self.dataset.filtered:
 			def var_map(thread_index, i1, i2, *blocks):
 				mask = self.dataset.evaluate_selection_mask("default" if self.is_masked else None, i1=i1, i2=i2)
 				if means is not None:
@@ -1177,7 +1177,7 @@ class SubspaceLocal(Subspace):
 			task = TaskMapReduce(self.dataset, self.expressions, covar_map, covars_reduce, remove_counts_and_normalize, info=True)
 			return self._task(task)
 		if means is None:
-			if self.async:
+			if self.delay:
 				means_wrapper = [None]
 				def do_vars(means):
 					means_wrapper[0] = means
@@ -1191,7 +1191,7 @@ class SubspaceLocal(Subspace):
 				return do_correlation(means, vars)
 		else:
 			if vars is None:
-				if self.async:
+				if self.delay:
 					def do_correlation_wrapper(vars):
 						return do_correlation(means, vars)
 					return self.vars(means=means).then(do_correlation_wrapper)
@@ -1209,7 +1209,7 @@ class SubspaceLocal(Subspace):
 		nansum = lambda x: np.nansum(x, dtype=np.float64)
 		# TODO: we can speed up significantly using our own nansum, probably the same for var and mean
 		nansum = vaex.vaexfast.nansum
-		if self.is_masked or self.dataset.selection_global:
+		if self.is_masked or self.dataset.filtered:
 			task = TaskMapReduce(self.dataset,\
 								 self.expressions, lambda thread_index, i1, i2, *blocks: [nansum(block[self.dataset.evaluate_selection_mask("default" if self.is_masked else None, i1=i1, i2=i2)])
 																						  for block in blocks],\
@@ -1262,10 +1262,11 @@ class SubspaceLocal(Subspace):
 		else:
 			histogram_done = Task.fulfilled(grid)
 		mutual_information_promise = histogram_done.then(vaex.kld.mutual_information)
-		return mutual_information_promise if self.async else mutual_information_promise.get()
+		return mutual_information_promise if self.delay else mutual_information_promise.get()
 
 
 	def limits_percentage(self, percentage=99.73, square=False):
+		import scipy.ndimage
 		limits = []
 		for expr in self.expressions:
 			subspace = self.dataset(expr)
@@ -1285,7 +1286,7 @@ class SubspaceLocal(Subspace):
 		return limits
 
 	def limits_sigma(self, sigmas=3, square=False):
-		if self.async:
+		if self.delay:
 			means_wrapper = [None]
 			def do_vars(means):
 				means_wrapper[0] = means
