@@ -17,8 +17,8 @@ def from_dict(d):
     if cls_name not in registry:
         raise ValueError('unknown class: ' + cls_name)
     else:
-        obj = registry[cls_name]()
-        obj.state_set(d['state'])
+        obj = registry[cls_name].state_from(d['state'])
+        #obj.state_set(d['state'])
         return obj
 
 def can_serialize(object):
