@@ -281,7 +281,12 @@ class ProgressWebSocket(tornado.websocket.WebSocketHandler):
 				response["job_phase"] = "EXCEPTION"
 			meta_data.update(response)
 			data_response = b""
-		text_response = json.dumps(meta_data).encode("utf8")\
+		try:
+			text_response = json.dumps(meta_data).encode("utf8")\
+						+ b"\n" + data_response
+		except Exception as e:
+			meta_data = dict(job_id=job_id, job_phase='EXCEPTION', **exception(e))
+			text_response = json.dumps(meta_data).encode("utf8")\
 						+ b"\n" + data_response
 
 		self.write_message(text_response, binary=True)
@@ -312,7 +317,7 @@ def error(msg):
 def process(webserver, user_id, path, fraction=None, progress=None, **arguments):
 	if not hasattr(webserver.thread_local, "executor"):
 		logger.debug("creating thread pool and executor")
-		webserver.thread_local.thread_pool = vaex.multithreading.ThreadPoolIndex(nthreads=webserver.threads_per_job)
+		webserver.thread_local.thread_pool = vaex.multithreading.ThreadPoolIndex(max_workers=webserver.threads_per_job)
 		webserver.thread_local.executor = vaex.execution.Executor(thread_pool=webserver.thread_local.thread_pool)
 		webserver.thread_pools.append(webserver.thread_local.thread_pool)
 
@@ -333,11 +338,11 @@ def process(webserver, user_id, path, fraction=None, progress=None, **arguments)
 		elif parts[0] == "datasets":
 			if len(parts) == 1:
 				#
-				response = dict(result=[{"name":ds.name, "full_length":ds.full_length(), "column_names":ds.get_column_names(strings=True),
-										 "description": ds.description, "descriptions":ds.descriptions,
-										 "ucds":ds.ucds, "units":{name:str(unit) for name, unit in ds.units.items()},
-										 "dtypes":{name:str(ds.columns[name].dtype) for name in ds.get_column_names(strings=True)},
-										 "virtual_columns":dict(ds.virtual_columns), "selection_global": ds.selection_global
+				response = dict(result=[{'name':ds.name,
+										'length_original':ds.length_original(),
+										'column_names':ds.get_column_names(strings=True),
+										 'dtypes':{name:str(ds.columns[name].dtype) for name in ds.get_column_names(strings=True)},
+										 'state': ds.state_get()
 										 } for ds in webserver.datasets])
 				logger.debug("response: %r", response)
 				return response
@@ -356,50 +361,17 @@ def process(webserver, user_id, path, fraction=None, progress=None, **arguments)
 							expressions = None
 						# make a shallow copy, such that selection and active_fraction is not shared
 						dataset_original = webserver.datasets_map[dataset_name]
-						dataset = dataset_original.shallow_copy(virtual=False)
-						if dataset_original.selection_global:
-							# copy the selection
-							selection = dataset_original.get_selection(dataset_original.selection_global)
-							dataset.set_selection(selection, name=dataset_original.selection_global)
-
-							# copy the cache
-							cache_for_global = dataset_original._selection_mask_caches[dataset_original.selection_global]
-							dataset._selection_mask_caches[dataset_original.selection_global] = dict(cache_for_global)
-							logger.info("set selection: %r to %r" % (dataset_original.selection_global, selection))
+						dataset = dataset_original.copy()
+						# copy the cache, 2 level deep clone
+						# currently not working, could be a good optimization
+						# dataset._selection_mask_caches = {key:dict(value) for key, value in dataset_original._selection_mask_caches.items()}
 						logger.info("method: %r args: %r" % (method_name, arguments))
 						# TODO: executor should be an argument to the stats functions..
 						dataset.executor = webserver.thread_local.executor
 						if dataset.mask is not None:
 							logger.debug("selection: %r", dataset.mask.sum())
-						if "active_fraction" in arguments:
-							active_fraction = arguments["active_fraction"]
-							logger.debug("setting active fraction to: %r", active_fraction)
-							dataset.set_active_fraction(active_fraction)
-						else:
-							if fraction is not None:
-								dataset.set_active_fraction(fraction)
-								logger.debug("auto fraction set to %f", fraction)
-						if "active_start_index" in arguments and "active_end_index" in arguments:
-							i1, i2 = arguments["active_start_index"], arguments["active_end_index"]
-							logger.debug("setting active range to: %r", (i1, i2))
-							dataset.set_active_range(i1, i2)
-
-						if "variables" in arguments:
-							variables = arguments["variables"]
-							logger.debug("setting variables to: %r", variables)
-							for key, value in variables:
-								dataset.set_variable(key, value)
-						if "virtual_columns" in arguments:
-							virtual_columns = arguments["virtual_columns"]
-							logger.debug("setting virtual_columns to: %r", virtual_columns)
-							for key, value in virtual_columns:
-								dataset.add_virtual_column(key, value)
-							for key, value in virtual_columns:
-								try:
-									dataset.validate_expression(value)
-								except (SyntaxError, KeyError, NameError) as e:
-									logger.exception("state was: %r", arguments)
-									return exception(e)
+						if 'state' in arguments:
+							dataset.state_set(arguments['state'])
 						if expressions:
 							for expression in expressions:
 								try:
@@ -407,31 +379,29 @@ def process(webserver, user_id, path, fraction=None, progress=None, **arguments)
 								except (SyntaxError, KeyError, NameError) as e:
 									return exception(e)
 						subspace = dataset(*expressions, executor=webserver.thread_local.executor) if expressions else None
-						if subspace:
-							# old stype selection
-							if "selection" in arguments:
-								selection_values = arguments["selection"]
-								if selection_values:
-									selection = vaex.dataset.selection_from_dict(dataset, selection_values)
-									dataset.set_selection(selection, executor=webserver.thread_local.executor)
-						else:
-							if "selections" in arguments:
-								selection_values = arguments["selections"]
-								if selection_values:
-									for name, value in selection_values.items():
-										if value:
-											selection = vaex.dataset.selection_from_dict(dataset, value)
-											dataset.set_selection(selection, name=name)
+						if dataset_original.has_selection(vaex.dataset.FILTER_SELECTION_NAME):
+							selection_original = dataset_original.get_selection(vaex.dataset.FILTER_SELECTION_NAME)
+							if not dataset.has_selection(vaex.dataset.FILTER_SELECTION_NAME):
+								dataset.set_selection(selection_original, name=vaex.dataset.FILTER_SELECTION_NAME)
+							else:
+								selection = dataset.get_selection(vaex.dataset.FILTER_SELECTION_NAME)
+								# UGLY: we 'stitch' the current selection at the end of the
+								# selection 'list'
+								selection_end = selection
+								while selection_end.previous_selection:
+									selection_end = selection_end.previous_selection
+								selection_end.previous_selection = selection_original
+								selection_end.mode = "and"
 						try:
 							if subspace:
-								if "selection" in arguments:
+								if 'selection' in arguments and arguments['selection']:
 									subspace = subspace.selected()
 							if subspace is None:
-								for name in "job_id expressions active_fraction selections variables virtual_columns active_start_index active_end_index".split():
+								for name in "job_id state auto_fraction expressions active_fraction selections variables virtual_columns active_start_index active_end_index".split():
 									arguments.pop(name, None)
 							else:
 								if subspace is not None:
-									for name in "job_id expressions active_fraction selection selections variables virtual_columns active_start_index active_end_index".split():
+									for name in "job_id state auto_fraction expressions active_fraction selection selections variables virtual_columns active_start_index active_end_index".split():
 										arguments.pop(name, None)
 							logger.debug("subspace: %r", subspace)
 							if subspace is None and method_name in "count cov correlation covariance mean std minmax min max sum var".split():
@@ -469,8 +439,7 @@ def process(webserver, user_id, path, fraction=None, progress=None, **arguments)
 								return ({"result": result})
 							elif method_name in ["evaluate"]:
 								result = task_invoke(dataset, method_name, **arguments)
-								result = result.tolist() if hasattr(result, "tolist") else result
-								return ({"result": result})
+								return result
 							else:
 								logger.error("unknown method: %r", method_name)
 								return error("unknown method: " + method_name)
@@ -591,7 +560,7 @@ class WebServer(threading.Thread):
 		#self.ioloop.close()
 		#self.ioloop.clear_current()
 		for thread_pool in self.thread_pools:
-			thread_pool.close()
+			thread_pool.shutdown()
 
 defaults_yaml = """
 address: 0.0.0.0
